from sqlalchemy import Column, DateTime, Integer, String

from peek_plugin_branch._private.storage.DeclarativeBase import DeclarativeBase
from peek_plugin_branch.tuples.BranchDetailTuple import BranchDetailTuple


class BranchDetailTable(DeclarativeBase):
    __tablename__ = "BranchDetail"

    id = Column(Integer, primary_key=True, autoincrement=True)

    modelSetKey = Column(String, nullable=False)

    key = Column(String, nullable=False)

    name = Column(String, nullable=False)

    comment = Column(String)

    userName = Column(String, nullable=False)

    updatedDate = Column(DateTime(timezone=True), nullable=False)

    createdDate = Column(DateTime(timezone=True), nullable=False)

    def toTuple(self) -> BranchDetailTuple:
        branch = BranchDetailTuple()
        for fieldName in BranchDetailTuple.tupleFieldNames():
            setattr(branch, fieldName, getattr(self, fieldName))
        return branch

    @staticmethod
    def fromTuple(inTuple: BranchDetailTuple) -> "BranchDetailTable":
        branch = BranchDetailTable()
        for fieldName in BranchDetailTuple.tupleFieldNames():
            setattr(branch, fieldName, getattr(inTuple, fieldName))
        return branch



from vortex.Tuple import addTupleType, TupleField
from vortex.TupleAction import TupleActionABC

from peek_plugin_branch._private.PluginNames import branchTuplePrefix
from peek_plugin_branch.tuples.BranchDetailTuple import BranchDetailTuple


@addTupleType
class CreateBranchActionTuple(TupleActionABC):
    __tupleType__ = branchTuplePrefix + "CreateBranchActionTuple"

    branchDetail: BranchDetailTuple = TupleField()



branchPluginName = "peek_plugin_branch"
branchFilt = {"plugin": "peek_plugin_branch"}
branchTuplePrefix = "peek_plugin_branch."
branchObservableName = "peek_plugin_branch"
branchActionProcessorName = "peek_plugin_branch"
branchTupleOfflineServiceName = "peek_plugin_branch"



from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

from peek_plugin_branch._private.PluginNames import branchFilt
from peek_plugin_branch._private.PluginNames import branchObservableName
from peek_plugin_branch.tuples.BranchDetailTuple import BranchDetailTuple
from .tuple_providers.BranchDetailTupleProvider import BranchDetailTupleProvider


def makeTupleDataObservableHandler(ormSessionCreator):
    """ " Make Tuple Data Observable Handler

    This method creates the observable object, registers the tuple providers and then
    returns it.

    :param ormSessionCreator: A function that returns a SQLAlchemy session when called

    :return: An instance of :code:`TupleDataObservableHandler`

    """
    tupleObservable = TupleDataObservableHandler(
        observableName=branchObservableName, additionalFilt=branchFilt
    )

    # Register TupleProviders here
    tupleObservable.addTupleProvider(
        BranchDetailTuple.tupleName(),
        BranchDetailTupleProvider(ormSessionCreator),
    )
    return tupleObservable



from txhttputil.util.ModuleUtil import filterModules


def loadPrivateTuples():
    """Load Private Tuples

    In this method, we load the private tuples.
    This registers them so the Vortex can reconstructed them from
    serialised data.

    """
    for mod in filterModules(__name__, __file__):
        __import__(mod, locals(), globals())



from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_plugin_branch._private.PluginNames import branchFilt
from peek_plugin_branch._private.PluginNames import branchActionProcessorName
from vortex.handler.TupleActionProcessorProxy import TupleActionProcessorProxy


def makeTupleActionProcessorProxy():
    return TupleActionProcessorProxy(
        tupleActionProcessorName=branchActionProcessorName,
        proxyToVortexName=peekServerName,
        additionalFilt=branchFilt,
    )
